#include "resu.h"

bool Alterable::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e1: ofstream saveFile( " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

bool Alterable::readingToProduceArray() {
    using namespace std;

// Reading list_of_items to produce an array
    ifstream in( list_of_items.c_str() );
    if( in.fail() ) {
        error = "e2: ifstream in( " + list_of_items + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string *p = new string("");
    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);

        if( p->length() > 0 ) {
            item.push_back(*p);
        } else {
            item.push_back(BELL);
        }

    }

    in.close();
    delete p;
    return true;
}

bool Alterable::searchingItem(std::string & s) {
    using namespace std;

    bool found = false;
    int q;

    for(ITEM::iterator i = item.begin(); i != item.end(); ++i) {
        q = s.find(*i);
        if(q != string::npos) {
            found = true;
            break;
        }

    }

    return found;
}

bool Alterable::searchingItemSequence(std::string & s) {
    using namespace std;

    bool found = false;
    int q;

    for(ITEM::iterator i = item.begin(); i != item.end(); ++i) {
        q = s.find(*i);
        if(q != string::npos) {
            found = true;
            sequence = *i;
            break;
        }

    }

    return found;
}

bool ASCIIdates::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e3: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            *buffer << asciiOnly(line) << NEW_LINE;
        }

    } else {
        error = "e4(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int q, r;
    string head, s, tail;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            q = s.rfind(TOKOBI);
            if(q != string::npos) {
                head = s.substr(0, q);
                q += TOKOBI_SIZE;
                tail = s.substr(q);
                s = head + tail;
            }

            *buffer << asciiOnly(s) << NEW_LINE;
        }

    }

    in.close();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool Combo::acquireFromFile(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e5: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string s;
    string *p = new string("");
    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        add(s);
    }

    delete p;
    return true;
}

bool BetweenColons::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e6: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e7: in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p);
    int r;
    string s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s);
        r = lineType->dateLine();
        if(r > 0) {
            listing();

            s = lineType->interposed();
            clear(s);
            continue;
        }

        add(s);
    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool Combo::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e8: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

        loto = checkLine1(line);
        switch(loto) {
        case ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT:
            error = "e9: ADD THE DRAW TO LINE-1 OF WHO.TXT";
            return false;
        case TOO_FEW_GAPS:
            error = "e10: TOO FEW GAPS";
            return false;
        }

    } else {
        error = "e11(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    bool found;
    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int loto, r;
    string combinationName("cmb7654-32-10.txt");
    string s("");
    TimeName *ct = new TimeName(COMBINATION, TEXT_FILE);
    ToFive *e = new ToFive(combinationName, s);
    ToSeven *n = new ToSeven(combinationName, s);
    ToSix *x = new ToSix(combinationName, s);

    produceArrays();
    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing();
            clear(s);
            continue;
        }

        found = searchingItemSequence(s);
        if(found) {
            delete ct;
            ct = new TimeName(COMBINATION, TEXT_FILE);
            combinationName = ct->temporaryName(SERIAL_NUMBER);

            s = firstNumberTrimmed(s);
            loto = ballCount[sequence];
            switch(loto) {
            case MINILOTO:
                delete e;
                e = new ToFive(combinationName, s);
                result = e->writeFile();
                if(result == FALSE) {
                    error = e->inquiry();
                    delete ct; delete e; delete lineType; delete p; 
                    return false;
                }

                delete e;
                break;
            case LOTO6:
                delete x;
                x = new ToSix(combinationName, s);
                result = x->writeFile();
                if(result == FALSE) {
                    error = x->inquiry();
                    delete ct; delete lineType; delete p; delete x; 
                    return false;
                }

                delete x;
                break;
            case LOTO7:
                delete n;
                n = new ToSeven(combinationName, s);
                result = n->writeFile();
                if(result == FALSE) {
                    error = n->inquiry();
                    delete ct; delete lineType; delete n; delete p; 
                    return false;
                }

                delete n;
                break;
            }

            acquireFromFile(combinationName);
        } else {
            add(s);
        }

    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete ct;
    delete lineType;
    delete p;
    return true;
}

bool CombosDates::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e12: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

        loto = checkLine1(line);
        switch(loto) {
        case ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT:
            error = "e13: ADD THE DRAW TO LINE-1 OF WHO.TXT";
            return false;
        case TOO_FEW_GAPS:
            error = "e14: TOO FEW GAPS";
            return false;
        case MINILOTO:
            all = 31;
            break;
        case LOTO6:
            all = 43;
            break;
        case LOTO7:
            all = 37;
            break;
        }

        settingArrays();
    } else {
        error = "e15(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing(NEW_LINE);
            clear(s);
            continue;
        }

        if( countBallsMesureSpan(s) ) {
            add(s);
        }

    }

    in.close();
    listing(NEW_LINE);
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool CombosDates::countBallsMesureSpan(std::string & s) {
    using namespace std;

    int c = 0;
    int q;

// Counting
    resettingPlaceArray();
    for(int i = 1; i <= all; ++i) {
        q = s.find(ball[i]);
        place[i] = q;
        if(q != string::npos) {
            c += 1;
        }

    }

    if(c != loto) {
        error = "e16: c != loto";
        return false;
    }

// Sorting places
    sort( place.begin(), place.end() );
    for(int i = 1; i <= all; ++i) {
        q = place[i];
        if(q == string::npos) {
            continue;
        } else if(q == 0) {
            continue;
        } else {
            span = q - place[ i - 1 ];
            if(span < NEIGHBOR) {
                error = "e17: span < NEIGHBOR";
                return false;
            }

        }

    }

    return true;
}

bool DateFollowedByName::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e18: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e19: in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p);
    int r;
    string s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s);
        r = lineType->dateLine();
        if(r > 0) {
            listing();

            s = lineType->rebuild();
            clear(s);
            continue;
        }

        add(s);
    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool Drafo::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e20: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
// Line-1
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e21(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    readingCsvToProduceArrays();
    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing();
            clear(s);
            continue;
        }

        s = GAP + s;
        s = checkOneByOne(s); // Replacing "V" with "7"
        s = replaceDraftWithFormal(s); // Replacing " 7 " with " 07 " for example
        s = s.substr(1);
        add(s);
    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool Drafo::readingCsvToProduceArrays() {
    using namespace std;

// Reading list_of_items to produce an array
    ifstream in( list_of_items.c_str() );
    if( in.fail() ) {
        error = "e22: ifstream in( " + list_of_items + CLOSE_BRACKET;
        return false;
    }

    bool secondField = false;
    bool yen = false;
    char *from;
    char *p;
    char *b;
    char c[TO_BUFFER_SIZE];
    char to[TO_BUFFER_SIZE];
    const int COMMA = ',';
    const int YEN = 0x5C;
    const int YEN_SIZE = 1;
    int comma1 = 0;
    int comma2;
    int lineLength;
    string *d = new string("");
    string *f = new string("");

    while( in.getline(c, TO_BUFFER_SIZE) ) {
        lineLength = strlen(c);
        for(int i = 0; i < lineLength; ++i) {
            if(yen) {
                yen = false;
                continue;
            }

            if(secondField) {
                if(c[i] == YEN) { // If YEN precedes a comma in CSV, the second field shall include the comma.
                    yen = true;
                } else if(c[i] == COMMA) {
                    comma2 = i;

                    c[comma2] = '\0';
                    from = & c[ comma1 + 1 ];
                    b = strchr(from, YEN);
                    if(b == NULL) {
                        strcpy(to, from);
                    } else {
                        *b = '\0';
                        strcpy(to, from);
                        b += YEN_SIZE; // The second field shall exclude YEN.
                        strcat(to, b);
                    }

                    delete f;
                    f = new string(to);
                    if( f->length() > 0 ) {
                        formal.push_back(*f);
                    } else {
                        formal.push_back(BELL);
                    }

                    secondField = false;
                }

            } else { // first field
                if(c[i] == YEN) { // If YEN precedes a comma in CSV, the first field shall include the comma.
                    yen = true;
                } else if(c[i] == COMMA) {
                    comma1 = i;

                    c[comma1] = '\0';
                    p = & c[0];
                    b = strchr(p, YEN);
                    if(b == NULL) {
                        strcpy(to, p);
                    } else {
                        *b = '\0';
                        strcpy(to, p);
                        b += YEN_SIZE; // The first field shall exclude YEN.
                        strcat(to, b);
                    }

                    delete d;
                    d = new string(to);
                    if( d->length() > 0 ) {
                        draft.push_back(*d);
                    } else {
                        draft.push_back(BELL);
                    }

                    secondField = true;
                }

            }

        }

    }

    in.close();
    delete d; delete f; 
    return true;
}

bool Dates::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e23: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            *buffer << line << NEW_LINE;
        }

    } else {
        error = "e24(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            *buffer << s << NEW_LINE;
        }

    }

    in.close();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool ExcludeText::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e25: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
// Line-1
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e26(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    bool through;
    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    readingToProduceArray();
    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing();
            clear(s);
            continue;
        }

        through = searchingItem(s);
        if(through) {
            while( in.getline(c, BUFFER_SIZE) ) {
                delete p;
                p = new string(c);
                s = *p;

                delete lineType;
                lineType = new DateFollowed(s, NO_SPLITTING);
                r = lineType->dateLine();
                if(r > 0) {
                    break;
                }

            }

            if( in.eof() || in.fail() ) { // at EOF 
                clear(empty);
                break;
            } else { // before EOF 
                clear(s);
                continue;
            }

        } else {
            add(s);
        }

    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool FormDateLine::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e27: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

    } else {
        error = "e28(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p);
    int q, r;
    string s = EMPTY;
    stringstream *os = new stringstream();

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

// ring buffer
    produceRingBuffer(RING_COUNT);

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        ring.pop_front();
        ring.push_back(*p);

        q = p->find(ID_COLON);
        if(q != string::npos) {
            delete os;
            os = new stringstream();
            RING::iterator i = ring.begin();
            *os << *i;
            ++i;
            for( ; i != ring.end(); ++i) {
                *os << *i << GAP;
            }

            s = os->str();
            delete lineType;
            lineType = new DateFollowed(s);
            r = lineType->dateLine();
            if(r > 0) {
                listing();

                s = lineType->interposed();
                clear(s);
                blankRingBuffer();
                continue;
            } else {
                RING::iterator i = ring.begin();
                s = *i;
            }

        } else {
            RING::iterator i = ring.begin();
            s = *i;
        }

        add(s);
    }

    in.close();

    ring.pop_front();
    for(RING::iterator i = ring.begin(); i != ring.end(); ++i) {
        add(*i);
    }

    listing();
    openSaveClose(saveName);

    delete lineType;
    delete os;
    delete p;
    return true;
}

bool NewLines::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e29: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    string empty(EMPTY);
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(empty);
        }

        loto = checkLine1(line);
        switch(loto) {
        case ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT:
            error = "e30: ADD THE DRAW TO LINE-1 OF WHO.TXT";
            return false;
        case TOO_FEW_GAPS:
            error = "e31: TOO FEW GAPS";
            return false;
        }

    } else {
        error = "e32(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    bool found;
    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string s;

    readingToProduceArray();
    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            listing(NEW_LINE);
            clear(s);
            continue;
        }

        found = searchingItemSequence(s);
        if(found) {
            delete lineType;
            lineType = new MultiLoto(s, sequence, NOT_AXIAL);
        } else {
            delete lineType;
            lineType = new Gaps(s, loto, NOT_AXIAL);
        }

        for(;;) {
            s = lineType->lotoLine();
            if( s.length() == 0 ) {
                break;
            }

            add(s);
        }

    }

    in.close();
    listing(NEW_LINE);
    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool QuoteRatherThanMark::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e33: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            *buffer << line << NEW_LINE;
        }

    } else {
        error = "e34(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

    string *p = new string("");
    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        s = replaceMARKwithBACK_QUOTE(s);
        *buffer << s << NEW_LINE;
    }

    in.close();
    openSaveClose(saveName);
    delete p;
    return true;
}

bool Segmentalize::altering(std::string & fileName) {
    using namespace std;

// Reading SEGMENT_TEXT
    ifstream segment_text( list_of_items.c_str() );
    if( segment_text.fail() ) {
        error = "e35: ifstream segment_text( " + list_of_items + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];
    int r;
    string *p = new string("");
    string modelLine, s, NOTHING_ERRONEOUS;
    Characterizable *lineType = new Consecutive(*p);

    if( segment_text.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        modelLine = *p;

        delete lineType;
        lineType = new Consecutive(modelLine);
        r = lineType->dateLine();
        if(r > 0) {
            NOTHING_ERRONEOUS = lineType->pileOfErrorMessages();
        } else {
            error = "e36(lineType): " + modelLine;
            delete lineType; delete p;
            return false;
        }

    } else {
        error = "e37: segment_text.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        delete lineType; delete p;
        return false;
    }

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName wt(WHO, TEXT_FILE);
        saveName = wt.temporaryName(serialNumber);
    }

// Reading WHO_TEXT
    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e38: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            add(line);
            listing();
            clear(modelLine);
        }

    } else {
        error = "e39(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    int line = 0;
    error = NOTHING_ERRONEOUS;
    if(matrix) {
        while( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            s = *p;

            add(s);
            line += 1;
            if(line >= period) {
                if( integrity_of_data(s) ) {
                    add(EMPTY); // NEW_LINE
                    s = lineType->rebuild();
                    add(s);
                    line = 0;
                }

            }

        }

    } else {
        while( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            s = *p;

            add(s);
            line += 1;
            if(line >= period) {
                add(EMPTY); // NEW_LINE
                s = lineType->rebuild();
                add(s);
                line = 0;
            }

        }

    }

    in.close();
    listing();
    openSaveClose(saveName);

    delete lineType; delete p;
    return true;
}

bool Segmentalize::integrity_of_data(std::string & p) {
    using namespace std;

    bool integrity = false;
    int q;

    string d = p;
    d += SENTINEL;
    int u = strcspn( d.c_str(), "0123456789");
    string ball = d.substr(u, 2);

    switch(row) {
    case 0:
        q = CELL_01_02_03_04_05.find(ball);
        if(q != string::npos) {
            row = 1;
            integrity = true;
        } else {
            row = 0;
            integrity = false;
        }

        break;
    case 1:
        q = CELL_16_17_18_19_20.find(ball);
        if(q != string::npos) {
            row = 2;
            integrity = true;
        } else {
            row = 0;
            integrity = false;
        }

        break;
    case 2:
        q = CELL_26_27_28_29_30.find(ball);
        if(q != string::npos) {
            row = NON_INDEX_NUMBER;
            integrity = true;
        } else {
            row = 0;
            integrity = false;
        }

        break;
    }

    if(integrity) {
        if(row == NON_INDEX_NUMBER) {
            row = 0;
        } else{
            integrity = false;
        }

    }

    return integrity;
}

bool TableReverseSorted::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e40: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];

    readingToProduceArray();
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            if(REMOVE_LINE_1 == item[0]) {
                ;
            } else {
                *buffer << line << NEW_LINE;
            }

        }

        loto = checkLine1(line);
        switch(loto) {
        case ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT:
            error = "e41: ADD THE DRAW TO LINE-1 OF WHO.TXT";
            return false;
        case TOO_FEW_GAPS:
            error = "e42: TOO FEW GAPS";
            return false;
        case MINILOTO:
            all = 31;
            break;
        case LOTO6:
            all = 43;
            break;
        case LOTO7:
            all = 37;
            break;
        }

        settingThreeArrays();
    } else {
        error = "e43(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int factor, m, n, r, sum;
    string previous, s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName *ti;

        if(BELL == item[1]) { // extention
            ti = new TimeName(TABLE, TEXT_FILE);
        } else {
            ti = new TimeName(TABLE, item[1]);
        }

        saveName = ti->temporaryName(serialNumber);
        delete ti;
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            continue;
        }

        s = countMeasureSort(s);
        if( s.length() > 0 ) {
            article.push_back(s);
        }

    }

    in.close();

// Sorting articles
    sort( article.begin(), article.end() );

    s = COLON; // If sorted, numbers precede colons.
    for(ITEM::iterator i = article.begin(); i != article.end(); ++i) {
        previous = s;
        s = *i;
        if(previous == s) {
            cout << s << endl;
            continue;
        }

        longList(s, " ", a);
        for(int k = 0; k < loto; ++k) {
            m = (int)a[k];
            sum = count[m];
            count[m] = sum + 1;
        }

    }

// reverse-sorting or not
    if(REVERSE == item[2]) { // reverse-sort
        factor = -1;
    } else {
        factor = 1;
    }

// Setting flags and typically copying negative numbers
    for(int i = 1; i <= all; ++i) {
        check[i] = true;
        countCopied[i] = factor * count[i];
    }

// Sorting
    sort( countCopied.begin(), countCopied.end() );

    for(int k = 0; k <= all; ++k) {
        n = factor * countCopied[k];

        for(int i = 1; i <= all; ++i) {
            if(count[i] == n) {
                if(check[i]) {
                    *buffer << ball[i] << item[3] << n << NEW_LINE;
                    check[i] = false;
                }

            }

        }

    }

    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

bool UniqueCombos::altering(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e44: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[BUFFER_SIZE];

    readingToProduceArray();
    if( in.getline(c, BUFFER_SIZE) ) {
        string line(c);
        if(firstLine) {
            if(REMOVE_LINE_1 == item[0]) {
                ;
            } else {
                *buffer << line << NEW_LINE;
            }

        }

        loto = checkLine1(line);
        switch(loto) {
        case ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT:
            error = "e45: ADD THE DRAW TO LINE-1 OF WHO.TXT";
            return false;
        case TOO_FEW_GAPS:
            error = "e46: TOO FEW GAPS";
            return false;
        case MINILOTO:
            all = 31;
            break;
        case LOTO6:
            all = 43;
            break;
        case LOTO7:
            all = 37;
            break;
        }

        settingArrays();
    } else {
        error = "e47(Line-1): in.getline(c, BUFFER_SIZE" + CLOSE_BRACKET;
        return false;
    }

    string *p = new string("");
    Characterizable *lineType = new DateFollowed(*p, NO_SPLITTING);
    int r;
    string previous, s;

    if(whoText) {
        saveName = WHO_TEXT;
    } else {
        TimeName *ti;

        if(BELL == item[1]) { // extention
            ti = new TimeName(UNIQUE, TEXT_FILE);
        } else {
            ti = new TimeName(UNIQUE, item[1]);
        }

        saveName = ti->temporaryName(serialNumber);
        delete ti;
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        s = *p;

        delete lineType;
        lineType = new DateFollowed(s, NO_SPLITTING);
        r = lineType->dateLine();
        if(r > 0) {
            continue;
        }

        s = countMeasureSort(s);
        if( s.length() > 0 ) {
            article.push_back(s);
        }

    }

    in.close();

// Sorting articles
    sort( article.begin(), article.end() );

    s = COLON; // If sorted, numbers precede colons.
    for(ITEM::iterator i = article.begin(); i != article.end(); ++i) {
        previous = s;
        s = *i;
        if(previous == s) {
            cout << s << endl;
            continue;
        }

        *buffer << s << NEW_LINE;
    }

    openSaveClose(saveName);

    delete lineType;
    delete p;
    return true;
}

Drafo::Drafo() {
    using namespace std;

    z["@"] = ' ';
    z["C"] = ' ';
    z["A"] = ' ';
    z["E"] = ' ';
    z["O"] = '0';
    z["P"] = '1';
    z["Q"] = '2';
    z["R"] = '3';
    z["S"] = '4';
    z["T"] = '5';
    z["U"] = '6';
    z["V"] = '7';
    z["W"] = '8';
    z["X"] = '9';
    z[""] = 'A';
    z[""] = 'L';
    z[""] = 'P';
    z[""] = 'Q';
    z["`"] = 'A';
    z["k"] = 'L';
    z["o"] = 'P';
    z["p"] = 'Q';
    z["b"] = '|';
    z["^"] = '/';

    buffer = new stringstream();
    combos = new stringstream();
    whose = EMPTY;
}

ExcludeText::ExcludeText() {
    using namespace std;

    buffer = new stringstream();
    combos = new stringstream();
    whose = EMPTY;
}

int Alterable::checkLine1(std::string & line) {
    using namespace std;

/*
012345678901234567890123456789012
31 18 15 06 14 25 
31 18 15 06 14 25 -A -776
35 29 12 16 13 28 05 
35 29 12 16 13 28 05 -B -548
04 26 32 29 05 25 15 27 11 
04 26 32 29 05 25 15 27 11 -D -92
*/
    int q;
    string s = line;
    for(int i = 0; i < 6; ++i) {
        q = s.find(GAP);
        if(q != string::npos) {
            q += GAP_SIZE;
            s = s.substr(q);
        } else {
            return TOO_FEW_GAPS;
        }

    }

    int lineLength = line.length();
    if(lineLength == 18) {
        return MINILOTO;
    }

    if( MINUS == line.substr(18, 1) ) {
        return MINILOTO;
    }

    if(lineLength == 21) {
        return LOTO6;
    }

    if( MINUS == line.substr(21, 1) ) {
        return LOTO6;
    }

    if(lineLength == 27) {
        return LOTO7;
    }

    if( MINUS == line.substr(27, 1) ) {
        return LOTO7;
    }

    return ADD_THE_DRAW_TO_LINE_1_OF_WHO_TXT;
}

int FormDateLine::produceRingBuffer(int size) {
    using namespace std;

    string BLANK(" ");
    ring.clear();
    for(int i = 0; i < size; i += 1) {
        ring.push_back(BLANK);
    }

    return ring.size();
}

int stringToInteger(std::string & data) {
    using namespace std;

    int value;
    stringstream ss(data);
    ss >> value;
    return value;
}

int zenkakuFragment(std::string & s, int mark_index, int mark) {
    using namespace std;

    bool zenkaku = false;
    char line[BUFFER_SIZE];
    const int AFTER_MARK = mark_index + 1;
    int c;
    int firstOrSecondByte = 0;
    int lineLength = s.length();
    unsigned char byte;

    strcpy(line, s.c_str() );
    for(int i = 0; i < lineLength && i < AFTER_MARK; ++i) {
        byte = (unsigned char)line[i];
        c = (int)byte;

        if(zenkaku) {
            firstOrSecondByte = SECOND_BYTE;
            if(i == mark_index) {
                break;
            } else {
                zenkaku = false;
                firstOrSecondByte = 0;
                continue;
            }

        }

        if(0x81 <= c && c <= 0xFC) {
            if(KANA_KUTEN <= c && c <= 0xDF) { // 
                zenkaku = false;
                firstOrSecondByte = 0;
            } else {
                zenkaku = true;
                firstOrSecondByte = FIRST_BYTE;
            }

        } else {
            zenkaku = false;
            firstOrSecondByte = 0;
            if(c == mark) {
                if(i == mark_index) {
                    break;
                }

            }

        }

    }

    return firstOrSecondByte;
}

NewLines::NewLines(): NOT_AXIAL(false) {
    using namespace std;

    buffer = new stringstream();
    combos = new stringstream();
    whose = EMPTY;
}

std::string Alterable::countMeasureSort(std::string & s) {
    using namespace std;

    int c = 0;
    int q;

// Counting
    resettingArrays();
    for(int i = 1; i <= all; ++i) {
        q = s.find(ball[i]);
        place[i] = q;
        if(q != string::npos) {
            c += 1;
            check[i] = true;
        }

    }

    if(c != loto) {
        error = "e48: c != loto";
        return EMPTY;
    }

// Sorting places
    sort( place.begin(), place.end() );
    for(int i = 1; i <= all; ++i) {
        q = place[i];
        if(q == string::npos) {
            continue;
        } else if(q == 0) {
            continue;
        } else {
            span = q - place[ i - 1 ];
            if(span < NEIGHBOR) {
                error = "e49: span < NEIGHBOR";
                return EMPTY;
            }

        }

    }

// Sorting balls
    stringstream os;
    for(int i = 1; i <= all; ++i) {
        if(check[i]) {
            os << ball[i] << GAP;
        }

    }

// Trimming
    string g = os.str();
    int gap = g.length() - 1;
    g = g.substr(0, gap);

    return g;
}

std::string Combo::firstNumberTrimmed(std::string & l) {
    using namespace std;

    int k = strcspn( l.c_str(), "0123456789");
    string c;
    string line = l.substr(k);
    int lineLength = line.length();

    for(int i = lineLength - 1; i >= 0; --i) {
        c = line.substr(i, 1);
        if(GAP == c) {
            continue;
        } else {
            ++i;
            line = line.substr(0, i);
            break;
        }

    }

    return line;
}

std::string Drafo::checkOneByOne(std::string & line) {
    using namespace std;

    bool zenkaku = false;
    char c[BUFFER_SIZE];
    char ascii[] = {'1', '\0'};
    char kanji[] = {'1', '2', '\0'};
    int i, k;
    int lineLength = line.length();
    string *p = new string("");
    stringstream os;
    unsigned char uc;

    strcpy(c, line.c_str() );
    for(k = 0; k < lineLength; ++k) {
        uc = (unsigned char)c[k];
        i = (int)uc;

        if(zenkaku) {
            zenkaku = false;
            kanji[1] = c[k];
            delete p;
            p = new string(kanji);
            ascii[0] = z[*p]; // typedef std::map<std::string, int> ZENKAKU_HANKAKU; ZENKAKU_HANKAKU z;
            if(ascii[0] == 0) {
                os << kanji;
            } else {
                os << ascii;
            }

            continue;
        }

        if(0x81 <= i && i <= 0xFC) {
            if(KANA_KUTEN <= i && i <= 0xDF) { // 
                zenkaku = false;
                switch(i) {
                case 0xA4: // 
                    os << GAP;
                    break;
                case 0xA5: // 
                    os << GAP;
                    break;
                default:
                    ascii[0] = c[k];
                    os << ascii;
                    break;
                }

            } else {
                zenkaku = true;
                kanji[0] = c[k];
            }

        } else {
            zenkaku = false;
            ascii[0] = c[k];
            os << ascii;
        }

    }

    delete p;
    return os.str();
}

std::string Drafo::replaceDraftWithFormal(std::string & s) {
    using namespace std;

    int q, thatLength;
    string head, tail, that;
    string line = s;

    ITEM::iterator k = formal.begin(); 
    for(ITEM::iterator i = draft.begin(); i != draft.end(); ++i, ++k) {
        that = *i;
        thatLength = that.length();

        tail = line;
        line = EMPTY;
        q = tail.find(that);
        while(q != string::npos) {
            head = tail.substr(0, q);
            q += thatLength;
            tail = tail.substr(q);

            line += head;
            line += *k;
            q = tail.find(that);
        }

        line += tail;
    }

    return line;
}

std::string ASCIIdates::asciiOnly(std::string & line) {
    using namespace std;

    bool kanji_size = false;
    char c[BUFFER_SIZE];
    char ascii[] = {'1', '\0'};
    char kanji[] = {'1', '2', '\0'};
    int b, i, k;
    int lineLength = line.length();
    string *p = new string("");
    stringstream os;
    unsigned char uc;

    strcpy(c, line.c_str() );
    for(k = 0; k < lineLength; ++k) {
        uc = (unsigned char)c[k];
        i = (int)uc;

        if(kanji_size) {
            kanji_size = false;
            kanji[1] = c[k];
            delete p;
            p = new string(kanji);
            b = YOBI_BEFORE_AFTER.find(*p);
            if(b != string::npos) {
                os << weekdays[ b/KANJI_SIZE ];
            }

            continue;
        }

        if(0x81 <= i && i <= 0xFC) {
            if(KANA_KUTEN <= i && i <= 0xDF) { // 
                kanji_size = false;
            } else {
                kanji_size = true;
                kanji[0] = c[k];
            }

        } else {
            kanji_size = false;
            ascii[0] = c[k];
            os << ascii;
        }

    }

    delete p;
    return os.str();
}

std::string QuoteRatherThanMark::replaceMARKwithBACK_QUOTE(std::string & line) {
    using namespace std;

    int lineLength = line.length();
    if(lineLength > MARK_INDEX) {
        string c = line.substr(MARK_INDEX, 1);
        if(c == MARK) {
            if(! zenkakuFragment(line, MARK_INDEX, MARK_C) ) {
                string head = line.substr(0, MARK_INDEX);
                string tail = line.substr( MARK_INDEX + MARK_SIZE );
                line = head + BACK_QUOTE + tail;
            }

        }

    }

    return line;
}

UniqueCombos::UniqueCombos(): un("unq") {
    using namespace std;

    buffer = new stringstream();
    UNIQUE = un;
}

TableReverseSorted::TableReverseSorted(): REVERSE("-1"), ta("tab") {
    using namespace std;

    buffer = new stringstream();
    TABLE = ta;
}

void Alterable::resettingArrays() {
    for(int i = 0; i <= all; ++i) {
        check[i] = false;
        count[i] = 0;
        place[i] = OMIT;
    }

}

void Alterable::resettingPlaceArray() {
    for(int i = 0; i <= all; ++i) {
        place[i] = OMIT;
    }

}

void Alterable::settingArrays() {
    using namespace std;

    for(int i = 0; i <= all; ++i) {
        count.push_back(0);
        place.push_back(OMIT);
    }

}

void Combo::produceArrays() { // vector, map
    using namespace std;

    item.push_back(BOX_6TO5); ballCount[BOX_6TO5] = 5;
    item.push_back(BOX_7TO5); ballCount[BOX_7TO5] = 5;
    item.push_back(BOX_8TO5); ballCount[BOX_8TO5] = 5;
    item.push_back(BOX_9TO5); ballCount[BOX_9TO5] = 5;
    item.push_back(BOX_7TO6); ballCount[BOX_7TO6] = 6;
    item.push_back(BOX_8TO6); ballCount[BOX_8TO6] = 6;
    item.push_back(BOX_9TO6); ballCount[BOX_9TO6] = 6;
    item.push_back(BOX_10TO6); ballCount[BOX_10TO6] = 6;
    item.push_back(BOX_8TO7); ballCount[BOX_8TO7] = 7;
    item.push_back(BOX_9TO7); ballCount[BOX_9TO7] = 7;
    item.push_back(BOX_10TO7); ballCount[BOX_10TO7] = 7;
    item.push_back(BOX_11TO7); ballCount[BOX_11TO7] = 7;
    itemCount = 12;
}

void FormDateLine::blankRingBuffer() {
    using namespace std;

    string BLANK(" ");
    ring.pop_front();
    ring.push_back(BLANK);
    ring.pop_front();
    ring.push_back(BLANK);
    ring.pop_front();
    ring.push_back(BLANK);
}

void TableReverseSorted::longList(std::string & s, const char *d, long *a) {
    char *p;
    char *stop;
    const int SIZE = s.length();
    char *line = new char[ SIZE + 1 ];

    strcpy(line, s.c_str() );
    p = strtok(line, d);
    *a = strtol(p, &stop, 10);

// Producing an array of long
    ++a;
    while (p != NULL) {
        p = strtok(NULL, " ");
        if (p != NULL) {
            *a = strtol(p, &stop, 10);
            ++a;
        }

    }

    delete line;
}

void TableReverseSorted::settingThreeArrays() {
    using namespace std;

    for(int i = 0; i <= all; ++i) {
        count.push_back(0);
        countCopied.push_back(0);
        place.push_back(OMIT);
    }

}
