#include "resu.h"

/*
resu C++ Version 1.35
Editing before finding lottery winners
Copyright 2017 Sogaya

GCC

*/

namespace {
    using namespace std;

    const bool WITH_COMMENTS = true;
    const bool WITHOUT_COMMENTS = false;
    const double VERSION = 1.35;
    const int ERROR_CODE = 1;
    const int PERIOD = 30;
    const string ASCII_DATE_LINES("-da");
    const string ASCII_DATES("-a");
    const string BETWEEN_COLONS("-b");
    const string COMBO("-c");
    const string COMBOS_DATES("-i");
    const string DATES("-d");
    const string DRAFO_CSV("drafo.csv");
    const string DRAFT_FORMAL("-z");
    const string EMPTY("");
    const string EXCLUDE("-e");
    const string EXCLUDE_TEXT("exclude.txt");
    const string MATRIX("-m");
    const string NEW_LINE("\n");
    const string NEW_LINE_CHARACTERS("-n");
    const string NO_REMOVING("-r-");
    const string OVERWRITE("-o");
    const string REMOVE_LINE_1("-1-");
    const string SEGMENT_TEXT("segmen.txt");
    const string SEGMENTALIZE("-s");
    const string SEPARATE_TEXT("separate.txt");
    const string SERIAL("-l");
    const string SIXTH("-6");
    const string TABULATE_REVERSE_SORT("-t");
    const string TABULATE_TEXT("tabulate.txt");
    const string THREE_LINES_UNITED("-3");
    const string UNIQUE("-u");
    const string UNIQUE_TEXT("unique.txt");
    const string VERSION_COMMENTS("-VC");
    const string VERSION_GUIDE("-V");
    const string WHENEVER("-w");
    const string WHO_TEXT("who.txt");
    const string ZOKO_FORM("-f");
}

bool removeTemporaryFiles() {
    using namespace std;

    char c[FILE_NAME_SIZE];
    ifstream in(REMOVE_ERROR_TEXT);
    if( in.fail() ) {
        return false;
    }

    while( in.getline(c, FILE_NAME_SIZE) ) {
        if( strcmp(c, WHO_TEXT.c_str() ) ) { // Do not remove WHO_TEXT.
            if( strcmp(c, TABULATE_TEXT.c_str() ) ) { // Do not remove TABULATE_TEXT.
                remove(c);
            }

        }

    }

    return true;
}

bool testDrafoText() {
    using namespace std;

    bool result;
    ifstream test( DRAFO_CSV.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( DRAFO_CSV.c_str() );
    file << "&#160;, ," << endl;
    file << "&nbsp;, ," << endl;
    file << "a,A," << endl;
    file << "l,L," << endl;
    file << "p,P," << endl;
    file << "q,Q," << endl;
    file << "\\,, ," << endl;
    file << "., ," << endl;
    file << "      , ," << endl;
    file << "     , ," << endl;
    file << "    , ," << endl;
    file << "   , ," << endl;
    file << "  , ," << endl;
    file << " 1 , 01 ," << endl;
    file << " 2 , 02 ," << endl;
    file << " 3 , 03 ," << endl;
    file << " 4 , 04 ," << endl;
    file << " 5 , 05 ," << endl;
    file << " 6 , 06 ," << endl;
    file << " 7 , 07 ," << endl;
    file << " 8 , 08 ," << endl;
    file << " 9 , 09 ," << endl;
    file << "ALLQP,ALL QP," << endl;
    file << "|,/," << endl;
    return result;
}

bool testExcludeText() {
    using namespace std;

    bool result;
    ifstream test( EXCLUDE_TEXT.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( EXCLUDE_TEXT.c_str() );
    file << "ʂ" << endl; // Shift_JIS
    file << " ID:" << endl;
    return result;
}

bool testSegmentText() {
    using namespace std;

    bool result;
    ifstream test( SEGMENT_TEXT.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( SEGMENT_TEXT.c_str() );
    file << "256 FLegend Xuthus F2017/08/14(Mon) 16:15:20.68 ID:BbkwMSDh" << endl;
    return result;
}

bool testSeparateText() {
    using namespace std;

    bool result;
    ifstream test( SEPARATE_TEXT.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( SEPARATE_TEXT.c_str() );
    file << "/" << endl;
    file << "|" << endl;
    return result;
}

bool testUniqueText() {
    using namespace std;

    bool result;
    ifstream test( UNIQUE_TEXT.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( UNIQUE_TEXT.c_str() );
    file << "-1-" << endl; // optional
    file << ".txt" << endl; // extention
    return result;
}

bool testTabulateText() {
    using namespace std;

    bool result;
    ifstream test( TABULATE_TEXT.c_str() );
    if( test.fail() ) {
        result = false;
    } else {
        test.close();
        result = true;
        return result;
    }

    ofstream file( TABULATE_TEXT.c_str() );
    file << "-1-" << endl; // optional
    file << ".txt" << endl; // extention
    file << "-1" << endl; // reverse-sort
    file << " | " << endl; // separator
    return result;
}

int main(int argc, char *argv[]) {
    using namespace std;

    Alterable *a = new ExcludeText(); 
    bool destroyWhoText = false;
    bool firstLine = true;
    bool matrix = false;
    bool remove_first = true;
    bool serialNumber = false;
    bool whoText = true;
    bool provided[PROVIDED_SIZE];
    int period = PERIOD;
    int q, result;
    Numbered *n = new Numbered("-w30");
    string *p = new string("");
    string fileName = WHO_TEXT;
    string list_of_items = EXCLUDE_TEXT;
    string argument;

    provided[0] = testDrafoText();
    provided[1] = testExcludeText();
    provided[2] = testSegmentText();
    provided[3] = testSeparateText();
    provided[4] = testUniqueText();
    provided[5] = testTabulateText();
    for(int i = 0; i < PROVIDED_SIZE; ++i) {
        if(provided[i]) {
            continue;
        } else {
            return ERROR_CODE;
        }

    }

    for(int i = 1; i < argc; ++i) {
        delete p;
        p = new string(argv[i]);
        argument = *p;

        if(ASCII_DATE_LINES == argument) {
            delete a;
            a = new ASCIIdates();
        } else if(ASCII_DATES == argument) {
            delete a;
            a = new ASCIIdates();
        } else if(BETWEEN_COLONS == argument) {
            delete a;
            a = new BetweenColons();
        } else if(COMBO == argument) {
            delete a;
            a = new Combo();
        } else if(COMBOS_DATES == argument) {
            delete a;
            a = new CombosDates();
        } else if(DATES == argument) {
            delete a;
            a = new Dates();
        } else if(DRAFT_FORMAL == argument) {
            list_of_items = DRAFO_CSV;
            delete a;
            a = new Drafo();
        } else if(EXCLUDE == argument) {
            list_of_items = EXCLUDE_TEXT;
            delete a;
            a = new ExcludeText();
        } else if(NEW_LINE_CHARACTERS == argument) {
            list_of_items = SEPARATE_TEXT;
            delete a;
            a = new NewLines();
        } else if(NO_REMOVING == argument) {
            remove_first = false;
        } else if(OVERWRITE == argument) {
            destroyWhoText = true;
        } else if(REMOVE_LINE_1 == argument) {
            firstLine = false;
        } else if(SEGMENTALIZE == argument) {
            list_of_items = SEGMENT_TEXT;
            delete a;
            a = new Segmentalize();
        } else if(SERIAL == argument) {
            serialNumber = true;
        } else if(SIXTH == argument) {
            delete a;
            a = new QuoteRatherThanMark();
        } else if(TABULATE_REVERSE_SORT == argument) {
            list_of_items = TABULATE_TEXT;
            delete a;
            a = new TableReverseSorted();
        } else if(THREE_LINES_UNITED == argument) { // The web browser may divide a date line into three lines, which shall be united to form the date line.
            delete a;
            a = new FormDateLine();
        } else if(UNIQUE == argument) {
            list_of_items = UNIQUE_TEXT;
            delete a;
            a = new UniqueCombos();
        } else if(VERSION_COMMENTS == argument) {
            guide(WITH_COMMENTS);
            delete n; delete p;
            return ERROR_CODE;
        } else if(VERSION_GUIDE == argument) {
            guide(WITHOUT_COMMENTS);
            delete n; delete p;
            return ERROR_CODE;
        } else if(ZOKO_FORM == argument) {
            delete a;
            a = new DateFollowedByName();
        } else {
            if(whoText) {
                fileName = argument;
                whoText = false;
            } else {
                q = argument.find(MATRIX);
                if(q != string::npos) {
                    delete n;
                    n = new Numbered(argv[i]);
                    period = n->number();
                    if( n->parameter() == MATRIX ) {
                        list_of_items = SEGMENT_TEXT;
                        matrix = true;
                        delete a;
                        a = new Segmentalize();
                        continue;
                    }

                    period = PERIOD;
                }

                q = argument.find(WHENEVER);
                if(q != string::npos) {
                    delete n;
                    n = new Numbered(argv[i]);
                    period = n->number();
                    if( n->parameter() == WHENEVER ) {
                        list_of_items = SEGMENT_TEXT;
                        matrix = false;
                        delete a;
                        a = new Segmentalize();
                        continue;
                    }

                    period = PERIOD;
                }

                list_of_items = argument;
            }

        }

    }

    delete n;

    if(remove_first) {
        removeTemporaryFiles();
    }

    if(whoText) {
        ;
    } else {
        whoText = destroyWhoText;
    }

    a->setting(list_of_items, period, firstLine, matrix, serialNumber, whoText);
    if( a->altering(fileName) ) {
        fileName = a->altered();
        cout << fileName << endl;
        result = 0;
    } else {
        cerr << a->inquiry() << endl;
        result = ERROR_CODE;
    }

    createRemoveErrorText();
    delete a;
    delete p;
    return result;
}

void guide(bool comments) {
    using namespace std;

    if(comments) {
        cerr << "resu C++ Version " << VERSION << NEW_LINE << endl 
            << "resu -V            # version " << endl 
            << "resu -VC           # version with comments " << endl 
            << "resu who.txt -1-   # Remove Line-1. " << endl 
            << "resu who.txt -3    # Unite three lines to form a date Line. " << endl 
            << "resu who.txt -6    # Replace carets with quotes. " << endl 
            << "resu who.txt -a    # ASCII date lines " << endl 
            << "resu who.txt -b    # Every name is interposed between zenkaku colons. " << endl 
            << "resu who.txt -c    # Expand 11-to-7 combinations. " << endl 
            << "resu who.txt -d    # date lines " << endl 
            << "resu who.txt -e exclude.txt  # Every nurupo resu is excluded from who.txt " << endl 
            << "resu who.txt -f    # Every date line is written in the zoko form. " << endl 
            << "resu who.txt -i    # Include combinations and date lines. " << endl
            << "resu who.txt -l    # serial number " << endl 
            << "resu who.txt -m30  # Ensure the integrity of each 3 x 3 matrix. " << endl 
            << "resu who.txt -n    # Replace slashes with new-line characters. " << endl 
            << "resu who.txt -o    # Overwrite who.txt " << endl 
            << "resu who.txt -r-   # Do not remove temporary files. " << endl 
            << "resu who.txt -t    # Tabulate and reverse-sort. " << endl
            << "resu who.txt -u    # unique lines " << endl
            << "resu who.txt -w30  # Add a date line whenever 30 lines appear. " << endl 
            << "resu who.txt -z    # Change characters from zenkaku to hankaku. " << endl;
    } else {
        cerr << "resu C++ Version " << VERSION << NEW_LINE << endl 
            << "resu -V            " << endl 
            << "resu -VC           " << endl 
            << "resu who.txt -1-   " << endl 
            << "resu who.txt -3    " << endl 
            << "resu who.txt -6    " << endl 
            << "resu who.txt -a    " << endl 
            << "resu who.txt -b    " << endl 
            << "resu who.txt -c    " << endl 
            << "resu who.txt -d    " << endl 
            << "resu who.txt -e exclude.txt  " << endl 
            << "resu who.txt -f    " << endl 
            << "resu who.txt -i    " << endl 
            << "resu who.txt -l    " << endl 
            << "resu who.txt -m30  " << endl 
            << "resu who.txt -n    " << endl 
            << "resu who.txt -o    " << endl 
            << "resu who.txt -r-   " << endl 
            << "resu who.txt -t    " << endl 
            << "resu who.txt -u    " << endl 
            << "resu who.txt -w30  " << endl 
            << "resu who.txt -z    " << endl;
    }

}
